import { CellRenderer, HyperlinkRenderer, ImageRenderer, TextRenderer } from '@lumino/datagrid';
import { Dict, ISerializers, WidgetModel, WidgetView } from '@jupyter-widgets/base';
import { BarRenderer } from './core/barrenderer';
import { VegaExprView } from './vegaexpr';
import { HtmlRenderer } from './core/htmlRenderer';
import { Scalar } from './utils';
declare type Scale = any;
declare type Processor = Scalar | VegaExprView | Scale;
interface ICellRendererAttribute {
    name: string;
    phosphorName: string | null;
    defaultValue: Scalar;
}
export declare abstract class CellRendererModel extends WidgetModel {
    defaults(): {
        _model_name: string;
        _model_module: any;
        _model_module_version: any;
        _view_name: string;
        _view_module: any;
        _view_module_version: any;
    };
    abstract get_attrs(): ICellRendererAttribute[];
    static model_name: string;
    static model_module: any;
    static model_module_version: any;
    static view_name: string;
    static view_module: any;
    static view_module_version: any;
}
export declare abstract class CellRendererView extends WidgetView {
    render(): Promise<void>;
    /**
     * Method that should be called when the theme has changed.
     */
    onThemeChanged(): Promise<void>;
    /**
     * Initialize the CellRenderer widget.
     *
     * @return The promise to initialize the renderer
     */
    protected initializeRenderer(): Promise<any>;
    private _on_some_processors_change;
    /**
     * Update the phosphor renderer value, and trigger an event so that the DataGrid widget knows it has
     * changed.
     */
    private updateRenderer;
    /**
     * Update the processor associated with the given name.
     *
     * @param name - The name of the attribute to process.
     *
     * @return The PromiseLike to update the processor view.
     */
    protected updateProcessor(name: string): any;
    /**
     * Process a cell attribute given the cell config.
     *
     * @param name - The name of the attribute to process.
     *
     * @param config - The configuration data for the cell.
     *
     * @param defaultValue - The default attribute value.
     */
    protected process(name: string, config: CellRenderer.CellConfig, defaultValue: Scalar): any;
    protected abstract createRenderer(options: TextRenderer.IOptions): CellRenderer;
    model: CellRendererModel;
    renderer: CellRenderer;
    processors: Dict<Processor>;
}
export declare class TextRendererModel extends CellRendererModel {
    defaults(): {
        _model_name: string;
        _view_name: string;
        font: string;
        text_color: null;
        text_wrap: boolean;
        text_elide_direction: string;
        text_value: null;
        background_color: null;
        vertical_alignment: string;
        horizontal_alignment: string;
        format: null;
        format_type: string;
        missing: string;
        _model_module: any;
        _model_module_version: any;
        _view_module: any;
        _view_module_version: any;
    };
    get_attrs(): ICellRendererAttribute[];
    static serializers: ISerializers;
    static model_name: string;
    static view_name: string;
}
export declare class TextRendererView extends CellRendererView {
    render(): Promise<void>;
    createRenderer(options: TextRenderer.IOptions): TextRenderer;
    getFormatter(options?: TextRenderer.formatGeneric.IOptions): TextRenderer.FormatFunc;
    renderer: TextRenderer;
    model: TextRendererModel;
}
export declare class BarRendererModel extends TextRendererModel {
    defaults(): {
        _model_name: string;
        _view_name: string;
        bar_color: string;
        bar_value: number;
        orientation: string;
        bar_vertical_alignment: string;
        bar_horizontal_alignment: string;
        show_text: boolean;
        font: string;
        text_color: null;
        text_wrap: boolean;
        text_elide_direction: string;
        text_value: null;
        background_color: null;
        vertical_alignment: string;
        horizontal_alignment: string;
        format: null;
        format_type: string;
        missing: string;
        _model_module: any;
        _model_module_version: any;
        _view_module: any;
        _view_module_version: any;
    };
    get_attrs(): ICellRendererAttribute[];
    static serializers: ISerializers;
    static model_name: string;
    static view_name: string;
}
export declare class BarRendererView extends TextRendererView {
    createRenderer(options: BarRenderer.IOptions): BarRenderer;
    renderer: BarRenderer;
    model: BarRendererModel;
}
export declare class HyperlinkRendererModel extends TextRendererModel {
    defaults(): {
        _model_name: string;
        _view_name: string;
        url: {};
        url_name: {};
        font: string;
        text_color: null;
        text_wrap: boolean;
        text_elide_direction: string;
        text_value: null;
        background_color: null;
        vertical_alignment: string;
        horizontal_alignment: string;
        format: null;
        format_type: string;
        missing: string;
        _model_module: any;
        _model_module_version: any;
        _view_module: any;
        _view_module_version: any;
    };
    get_attrs(): ICellRendererAttribute[];
    static serializers: ISerializers;
    static model_name: string;
    static view_name: string;
}
export declare class HyperlinkRendererView extends TextRendererView {
    createRenderer(options: HyperlinkRenderer.IOptions): HyperlinkRenderer;
    renderer: HyperlinkRenderer;
    model: HyperlinkRendererModel;
}
export declare class ImageRendererModel extends CellRendererModel {
    defaults(): {
        _model_name: string;
        _view_name: string;
        background_color: {};
        placeholder: {};
        text_color: {};
        width: {};
        height: {};
        _model_module: any;
        _model_module_version: any;
        _view_module: any;
        _view_module_version: any;
    };
    get_attrs(): ICellRendererAttribute[];
    static serializers: ISerializers;
    static model_name: string;
    static view_name: string;
}
export declare class ImageRendererView extends CellRendererView {
    createRenderer(options: ImageRenderer.IOptions): TextRenderer | ImageRenderer;
    renderer: ImageRenderer;
    model: ImageRendererModel;
}
export declare class HtmlRendererModel extends CellRendererModel {
    defaults(): {
        _model_name: string;
        _view_name: string;
        font: string;
        placeholder: string;
        text_color: null;
        background_color: null;
        vertical_alignment: string;
        horizontal_alignment: string;
        _model_module: any;
        _model_module_version: any;
        _view_module: any;
        _view_module_version: any;
    };
    get_attrs(): ICellRendererAttribute[];
    static serializers: ISerializers;
    static model_name: string;
    static view_name: string;
}
export declare class HtmlRendererView extends CellRendererView {
    createRenderer(options: HtmlRenderer.IOptions): any;
    renderer: any;
    model: HtmlRendererModel;
}
export {};
