import { Transform } from './transform';
import { ViewBasedJSONModel } from './viewbasedjsonmodel';
import { DataGrid } from '@lumino/datagrid';
import { DataModel } from '@lumino/datagrid';
import { Message } from '@lumino/messaging';
import { BoxPanel } from '@lumino/widgets';
import { VirtualElement } from '@lumino/virtualdom';
/**
 * An interactive widget to add filter transformations to the data model.
 */
export declare class InteractiveFilterDialog extends BoxPanel {
    /**
     * Construct a new InteractiveFilterDialog.
     *
     * @param options - The options for initializing the InteractiveFilterDialog.
     */
    constructor(options: InteractiveFilterDialog.IOptions);
    /**
     * Connects to the "Select All" widget signal and
     * toggles checking all/none of the unique elements
     * by adding/removing them from the state object
     */
    private _connectToCheckbox;
    /**
     * Checks for any undefined values in `this._filterValue`.
     *
     * Note: This should be expanded in the future to also check for dtype
     * inappropriate values.
     */
    hasValidFilterValue(): boolean;
    /**
     * Applies the active transformation to the linked data model.
     */
    applyFilter(): void;
    /**
     * Updates the DOM elements with transform state from the linked data model.
     */
    updateDialog(): void;
    /**
     * Renders the widget with VirtualDOM.
     */
    private _render;
    /**
     * Displays the unique values of a column.
     */
    _renderUniqueVals(): void;
    /**
     * Checks whether all unique elements in the column
     * are present as "selected" in the state. This
     * function is used to determine whether the
     * "Select all" button should be ticked when
     * opening the filter by value menu.
     */
    updateSelectAllCheckboxState(): void;
    /**
     * Open the menu at the specified location.
     *
     * @param options - The additional options for opening the menu.
     */
    open(options: InteractiveFilterDialog.IOpenOptions): void;
    /**
     * Handle the DOM events for the filter dialog.
     *
     * @param event - The DOM event sent to the panel.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the panel's DOM node. It should
     * not be called directly by user code.
     */
    handleEvent(event: Event): void;
    /**
     * Handle the `'mousedown'` event for the menu.
     *
     * #### Notes
     * This listener is attached to the document node.
     */
    protected _evtMouseDown(event: MouseEvent): void;
    /**
     * Handle the `'keydown'` event for the menu.
     *
     * #### Notes
     * This listener is attached to the menu node.
     */
    protected _evtKeyDown(event: KeyboardEvent): void;
    /**
     * A message handler invoked on a `'before-attach'` message.
     */
    protected onBeforeAttach(msg: Message): void;
    /**
     * A message handler invoked on an `'after-detach'` message.
     */
    protected onAfterDetach(msg: Message): void;
    /**
     * Creates the input dialog box for the filter-by-value
     * menu.
     */
    createTextInputDialog(): VirtualElement;
    /**
     * Creates a `VirtualElement` to display the menu title.
     */
    createTitleNode(): VirtualElement;
    /**
     * Creates a `VirtualElement` to display an input element with "apply" button.
     *
     * Note: The `key` is randomly assigned to ensure that this element is always
     * rerendered with current state. User interaction with `input` elements
     * can cause attribute changes that are not recognized by VirtualDOM.
     */
    createSingleValueNode(): VirtualElement;
    /**
     * Creates a `VirtualElement` to display an input element with "apply" button.
     *
     * Note: The `key` is randomly assigned to ensure that this element is always
     * rerendered with current state. User interaction with `input` elements
     * can cause attribute changes that are not recognized by VirtualDOM.
     */
    createDualValueNode(): VirtualElement;
    /**
     * Creates a `VirtualElement` to display a "loading" message.
     */
    protected createLoadingMessageNodes(): VirtualElement;
    /**
     * Creates a Promise that resolves to a `VirtualElement` to display the unique
     * values of a column.
     */
    protected createUniqueValueNodes(): Promise<VirtualElement>;
    /**
     * Creates a `VirtualElement` to display the available filter operators.
     *
     * Note: The `key` is randomly assigned to ensure that this element is always
     * rerendered with current state. User interaction with `input` elements
     * can cause attribute changes that are not recognized by VirtualDOM.
     */
    protected createOperatorList(): VirtualElement;
    /**
     * Creates a `VirtualElement` to display an "apply" button.
     */
    protected createApplyButtonNode(): VirtualElement;
    /**
     * Creates an array of VirtualElements to represent the available operators
     * for columns with a numerical dtype.
     */
    private _createNumericalOperators;
    /**
     * Creates an array of VirtualElements to represent the available operators
     * for columns with a date dtype.
     */
    private _createDateOperators;
    /**
     * Creates an array of VirtualElements to represent the available operators
     * for columns with a boolean dtype.
     */
    private _createBooleanOperators;
    /**
     * Creates an array of VirtualElements to represent the available operators
     * for columns with a categorical dtype.
     */
    private _createCategoricalOperators;
    addRemoveAllUniqueValuesToState(add: boolean): void;
    /**
     * Returns a reference to the data model used for this menu.
     */
    get model(): ViewBasedJSONModel;
    /**
     * Updates the data model used for this menu.
     */
    set model(model: ViewBasedJSONModel);
    /**
     * Returns the current input value of the dialog.
     */
    get value(): InteractiveFilterDialog.FilterValue;
    /**
     * Returns the currently active filter operator.
     */
    get operator(): Transform.FilterOperator;
    /**
     * Returns the active column index.
     */
    get region(): DataModel.CellRegion;
    /**
     * Returns the active Cellregion.
     */
    get columnIndex(): number;
    /**
     * Returns the active column dtype.
     */
    get columnDType(): string;
    private _model;
    private _columnDType;
    private _columnIndex;
    private _region;
    private _mode;
    private _filterOperator;
    private _filterValue;
    private _textInputFilterValue;
    private _uniqueValueGrid;
    private _filterByConditionWidget;
    private _titleWidget;
    private _textInputWidget;
    private _applyWidget;
    private _uniqueValueStateManager;
    hasFilter: boolean;
    userInteractedWithDialog: boolean;
    private _selectAllCheckbox;
}
/**
 * The namespace for the `InteractiveFilterDialog` class statics.
 */
export declare namespace InteractiveFilterDialog {
    /**
     * An options object for creating an `InteractiveFilterDialog`.
     */
    interface IOptions {
        model: ViewBasedJSONModel;
    }
    /**
     * A type alias for various filter modes.
     */
    type FilterMode = 'condition' | 'value';
    /**
     * Type alias for valid input element values.
     */
    type FilterValue = Transform.FilterValue | undefined | undefined[];
    /**
     * An options object for the `open` method of this item.
     */
    interface IOpenOptions {
        /**
         * The client X coordinate of the menu location.
         */
        x: number;
        /**
         * The client Y coordinate of the menu location.
         */
        y: number;
        /**
         * The CellRegion of the `cellClick` that triggered this call.
         */
        region: DataModel.CellRegion;
        /**
         * The column index of the `cellClick` that triggered this call.
         */
        columnIndex: number;
        /**
         * Disallow repositioning of the X coordinates to prevent the menu from
         * extending out of the window.
         */
        forceX: boolean;
        /**
         * Disallow repositioning of the Y coordinates to prevent the menu from
         * extending out of the window.
         */
        forceY: boolean;
        /**
         * Selects if widget will open in `condition` or `value` mode.
         */
        mode: FilterMode;
    }
}
/**
 * Manages the selection state of the grid that displays the unique values
 * of a column.
 */
export declare class UniqueValueStateManager {
    constructor(options: UniqueValueStateManager.IOptions);
    has(region: DataModel.CellRegion, columnIndex: number, value: any): boolean;
    getKeyName(region: DataModel.CellRegion, columnIndex: number): string;
    add(region: DataModel.CellRegion, columnIndex: number, value: any): void;
    remove(region: DataModel.CellRegion, columnIndex: number, value: any): void;
    getValues(region: DataModel.CellRegion, columnIndex: number): any[];
    private _state;
    private _grid;
}
/**
 * The namespace for the `UniqueValueStateManager` class statics.
 */
export declare namespace UniqueValueStateManager {
    /**
     * An options object for initializing an UniqueValueStateManager.
     */
    interface IOptions {
        /**
         * The DataGrid to manage selection state for
         */
        grid: DataGrid;
    }
}
/**
 * The namespace for the `UniqueValueGridMouseHandler` class statics.
 */
export declare namespace UniqueValueGridMouseHandler {
    /**
     * An options object for initializing a UniqueValueGridMouseHandler.
     */
    interface IOptions {
        /**
         * The state manager linked to the grid for this mouse handler.
         */
        stateManager: UniqueValueStateManager;
        /**
         * The dialog linked to this mouse handler.
         */
        dialog: InteractiveFilterDialog;
    }
}
