"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportedClass = void 0;
const coreutils_1 = require("@lumino/coreutils");
const datagrid_1 = require("@lumino/datagrid");
const signaling_1 = require("@lumino/signaling");
const utils_1 = require("../utils");
// Workaround for Jupyter Lab 3 / ipywidget 7 compatibility
let exportedClass;
exports.exportedClass = exportedClass;
/**
 * A cell renderer which renders data values as htmls.
 */
if (datagrid_1.AsyncCellRenderer) {
    exports.exportedClass = exportedClass = (_a = class HtmlRenderer extends datagrid_1.AsyncCellRenderer {
            /**
             * Construct a new html renderer.
             *
             * @param options - The options for initializing the renderer.
             */
            constructor(options = {}) {
                super();
                this._imgLoaded = new signaling_1.Signal(this);
                this.font = options.font || '12px';
                this.placeholder = options.placeholder || '...';
                this.textColor = options.textColor || utils_1.Theme.getFontColor();
                this.backgroundColor =
                    options.backgroundColor || utils_1.Theme.getBackgroundColor();
                this.verticalAlignment = options.verticalAlignment || 'bottom';
                this.horizontalAlignment = options.horizontalAlignment || 'left';
            }
            isReady(config) {
                return (!config.value || HtmlRenderer.dataCache.get(config.value) !== undefined);
            }
            get imageLoaded() {
                return this._imgLoaded;
            }
            async load(config) {
                if (!config.value) {
                    return;
                }
                const value = config.value;
                const loadedPromise = new coreutils_1.PromiseDelegate();
                HtmlRenderer.dataCache.set(value, undefined);
                const img = new Image();
                img.src = this.htmlToSvg(config);
                img.width = config.width;
                img.height = config.height;
                img.onload = () => {
                    HtmlRenderer.dataCache.set(value, img);
                    loadedPromise.resolve();
                    this._imgLoaded.emit();
                };
                return loadedPromise.promise;
            }
            paintPlaceholder(gc, config) {
                this.drawBackground(gc, config);
                this.drawPlaceholder(gc, config);
            }
            /**
             * Paint the content for a cell.
             *
             * @param gc - The graphics context to use for drawing.
             *
             * @param config - The configuration data for the cell.
             */
            paint(gc, config) {
                this.drawBackground(gc, config);
                this.drawImage(gc, config);
            }
            /**
             * Draw the background for the cell.
             *
             * @param gc - The graphics context to use for drawing.
             *
             * @param config - The configuration data for the cell.
             */
            drawBackground(gc, config) {
                // Resolve the background color for the cell.
                const color = datagrid_1.CellRenderer.resolveOption(this.backgroundColor, config);
                // Bail if there is no background color to draw.
                if (!color) {
                    return;
                }
                // Fill the cell with the background color.
                gc.fillStyle = color;
                gc.fillRect(config.x, config.y, config.width, config.height);
            }
            /**
             * Draw the placeholder for the cell.
             *
             * @param gc - The graphics context to use for drawing.
             *
             * @param config - The configuration data for the cell.
             */
            drawPlaceholder(gc, config) {
                const placeholder = datagrid_1.CellRenderer.resolveOption(this.placeholder, config);
                const color = datagrid_1.CellRenderer.resolveOption(this.textColor, config);
                const textX = config.x + config.width / 2;
                const textY = config.y + config.height / 2;
                // Draw the placeholder.
                gc.fillStyle = color;
                gc.fillText(placeholder, textX, textY);
            }
            /**
             * Draw the html.
             *
             * @param gc - The graphics context to use for drawing.
             *
             * @param config - The configuration data for the cell.
             */
            drawImage(gc, config) {
                // Bail early if there is nothing to draw
                if (!config.value) {
                    return;
                }
                const img = HtmlRenderer.dataCache.get(config.value);
                // If it's not loaded yet, show the placeholder
                if (!img) {
                    return this.drawPlaceholder(gc, config);
                }
                const font = datagrid_1.CellRenderer.resolveOption(this.font, config);
                const textColor = datagrid_1.CellRenderer.resolveOption(this.textColor, config);
                const textHeight = datagrid_1.TextRenderer.measureFontHeight(font);
                const vAlign = datagrid_1.CellRenderer.resolveOption(this.verticalAlignment, config);
                const hAlign = datagrid_1.CellRenderer.resolveOption(this.horizontalAlignment, config);
                const boxHeight = config.height - (vAlign === 'center' ? 1 : 2);
                if (boxHeight <= 0) {
                    return;
                }
                gc.textBaseline = 'bottom';
                gc.textAlign = hAlign;
                gc.font = font;
                gc.fillStyle = textColor;
                if (textHeight > boxHeight) {
                    gc.beginPath();
                    gc.rect(config.x, config.y, config.width, config.height - 1);
                    gc.clip();
                }
                gc.drawImage(img, config.x, config.y);
            }
            htmlToSvg(config) {
                const font = datagrid_1.CellRenderer.resolveOption(this.font, config);
                const textColor = datagrid_1.CellRenderer.resolveOption(this.textColor, config);
                const vAlign = datagrid_1.CellRenderer.resolveOption(this.verticalAlignment, config);
                const hAlign = datagrid_1.CellRenderer.resolveOption(this.horizontalAlignment, config);
                const width = datagrid_1.CellRenderer.resolveOption(config.width, config);
                const height = datagrid_1.CellRenderer.resolveOption(config.height, config);
                const text = datagrid_1.CellRenderer.resolveOption(config.value, config);
                const html = `<svg xmlns="http://www.w3.org/2000/svg" width="${width}px" height="${height}px">
      <foreignObject width="${width}px" height="${height}px">
        <div
          xmlns="http://www.w3.org/1999/xhtml"
          style="display: table-cell; font: ${font}; width: ${width}px; height: ${height}px; color: ${textColor}; vertical-align: ${vAlign === 'center' ? 'middle' : vAlign}; text-align: ${hAlign}"
        >
          <div style="display: inline-block; padding: 0 2px">${text}</div>
        </div>
      </foreignObject>
    </svg>`;
                return 'data:image/svg+xml,' + encodeURIComponent(html);
            }
        },
        _a.dataCache = new Map(),
        _a);
}
//# sourceMappingURL=htmlRenderer.js.map